from .baseApi import BaseAPI
import random
from .urls import URL
import base64
from .utils.jkingo import md5_hash_passwd, encode_param
from .referers import Rerfer
import re

class JwCrawl(BaseAPI):
    def __init__(self, xh=None, password=None):
        super(JwCrawl,self).__init__()
        self.urls = [
            "http://222.195.158.225/",
            "http://222.195.158.206/",
            "http://222.195.158.207/",
            "http://222.195.158.208/",
        ]
        self.host = ""
        self.sessionId = ""
        self.set_timeout(5)
        if xh and password:
            self.xh = xh
            self.login(xh, password)

    def test_host(self):
        num = 0
        while True:
            try:
                slen = len(self.urls)  # 每次都测一下长度，每次长度都会变化。
                if slen == 0:
                    self.host = ""
                    print("无可用接口")
                    break
                num = random.randrange(0, slen)  # 随机选择一个网站
                URL = self.urls[num] + "cas/logon.action"
                self.request(URL)
                if self.get_code() == 200:
                    self.host = self.urls[num]
                    self.sessionId = self.get_cookie("JSESSIONID")  # sessionid
                    break
                else:
                    del self.urls[num]  # 该网站不可用
            except:
                del self.urls[num]  # 该网站不可用

    def get_token(self):  # 使用正则比字符串处理慢了15倍，因此采用字符串处理的方式进行。
        url= URL.get_jw_token_url(self.host)
        content = self.request(url)
        content = content.split("\n")
        _deskey = content[2].replace("var _deskey = '","").replace("';\r","")
        _nowtime = content[3].replace("var _nowtime = '","").replace("';\r", "")
        # end = time.time()
        # print(end - now)
        # print(_deskey)
        # print(_nowtime)
        # d = re.findall("= (.+);", content)
        # _deskey = d[0].replace('\'', '')
        # _nowtime = d[1].replace('\'', '')
        # end = time.time()
        # print(end-now)
        data = {'_deskey': _deskey, '_nowtime': _nowtime}
        return data

    def login(self,username="", password="", verify=""):
        self.test_host() # 如果不需要token的话，目前这部是可用省略的，
        if not username and password:
            return None
        u = base64.b64encode((username + ";;" + self.sessionId).encode(encoding="gbk")).decode(encoding="gbk")
        post_data = {
            '_u': base64.b64encode((username + ";;" + self.sessionId).encode(encoding="gbk")),
            '_p': md5_hash_passwd(password, verify),
            'randnumber': verify.lower(),
            'isPasswordPolicy': '1',
        }
        data = self.request(URL.get_jw_login_url(self.host),post_data)
        return self.get_json(data)

    def get_info(self):
        """
        个人信息查询
        :return:{'csrq': ‘生日’, 'xm': '姓名', 'yx': '院系', 'nj': '年级', 'zy': '计算机科学与技术14', 'xh': '学号'}
        """
        post_data = {'hidOption': 'InitData'}
        data = self.request(URL.get_jw_info_url(self.host),post_data,Rerfer.get_jw_info_refer(self.host))
        info = self.get_xml(data).get("info")
        result = dict()
        result['xh'] = info.get("xh")
        result['nj'] = info.get("rxnj")
        result['csrq'] = info.get("csrq")
        result['xm'] = info.get("xm")
        result['zy'] = info.get("bjmc")
        result['yx'] = info.get("yxb")
        return result

    def get_person_score(self, xh="",xn="",xq=""):
        """
        查询个人成绩
        :param xh:
        :param xn:
        :param xq:
        :return:{'2017年春季学期': [
        {'序号': '1', '备注': '.', '取得方式': '初修取得', '课程类别': '必修课/工作技能', '综合成绩': '优秀',
        '课程': '[080504104999]毕业设计', '学分': '12.0', '修读性质': '初修', '考核方式': '论文'}]
        """
        if xn == "" or xq == "":
            param = "xn=2017&xn1=2016&xq=1&ysyx=yscj&sjxz=sjxz1&userCode=%s&ysyxS=on&sjxzS=on" % (xh)
        else:
            param = "xn=%s&xq=%s&ysyx=yscj&sjxz=sjxz3&userCode=%s&ysyxS=on&sjxzS=on" % (xn, xq, xh)
        token = self.get_token()
        post_data = encode_param(param,token)
        data = self.request(URL.get_jw_person_score_url(self.host), post_data, Rerfer.get_jw_person_score_refer(self.host))
        data = self.get_table(data)
        result = dict()
        for x in data:
            if len(x) == 1 and type(x[0]) == list:
                key = x[0][0].split("：")[1]
            else:
                result[key] = x
        return result

    def get_class_score(self,xn, xq, kcdm, xkh, type=1):
        """
        查询整个班级的成绩,type为原始成绩，默认为1
        :param xn:
        :param xq:
        :param kcdm:
        :param xkh:
        :param type:
        :return:
        """
        post_data = {
            'xn': xn,
            'xq': xq,
            'kcdm': kcdm,
            'skbjdm': xkh,
            'pxfs': 'xhsx',
        }
        data = self.request(URL.get_jw_class_score_url(self.host, type), post_data, Rerfer.get_jw_info_refer(self.host))
        return self.get_table(data)

    def get_student_course(self, xh, xn, xq):
        """
        查询个人选课
        :param xh:
        :param xn:
        :param xq:
        :return:
        """
        param = "xn=%s&xq=%s&xh=%s" % (xn, xq, xh)
        param = base64.b64encode(param.encode(encoding="gbk")).decode(encoding="gbk")
        data = self.request(URL.get_jw_student_course_url(self.host)+param,
                            referer=Rerfer.get_jw_student_course_refer(self.host))
        return self.get_table(data)

    def get_school_course(self, xn, xq, kcfw="Common"):
        """
        :param xn:
        :param xq:
        :param kcfw:
        :return:
        """
        post_data = {
            'initQry': '0',
            'xn': xn,
            'xq': xq,
            'items': '',
            'kcfw': kcfw,  # kcfw: Common Specialty PublicBasic
            'sel_nj': '',
            'sel_zydm': '',
            'sel_schoolarea': '',
            'sel_cddwdm': '',
            'sel_kc': '',
            'kcmc': '',
            "tableId": "6146"
        }
        count = 1
        max_page = -1
        courses = []
        while True:
            data = self.request(URL.get_jw_scholl_course_url(self.host,count), post_data,
                                Rerfer.get_jw_school_course_refer(self.host))
            if max_page == -1:
                page = (re.findall(r'reloadPage.+\)', data))
                page = re.search(r'(\d+),(\d+)', page[0])
                max_page = int(page.group(1)) + 1
            count += 1
            data = self.get_school_table(data)
            courses.extend(data)
            if count == max_page:
                break
        return data

    def get_student_test(self, xh, xn, xq, type):
        param = {
            'initQry': 0,
            'xh': xh,
            'xn': xn,
            'xq': xq,
            'xnxq': '%s-%s' % (xn, xq),
            'kslcdm': type,
        }
        html = self.request(URL.get_jw_student_test_url(self.host), param, Rerfer.get_jw_student_test_refer(self.host))
        return self.get_table(html)




